package com.crm.manager.service.risk;

import com.crm.rely.backend.core.dto.base.BaseResultDto;
import com.crm.rely.backend.core.dto.base.ResultWithPagerDto;
import com.crm.rely.backend.core.dto.risk.symbol.RiskSymbolDto;
import com.crm.rely.backend.core.dto.risk.symbol.record.RiskSymbolRecordDto;
import com.crm.rely.backend.core.entity.base.SingleLongEntity;
import com.crm.rely.backend.core.entity.risk.symbol.RiskSymbolAddEntity;
import com.crm.rely.backend.core.entity.risk.symbol.RiskSymbolDeleteEntity;
import com.crm.rely.backend.core.entity.risk.symbol.RiskSymbolSearchPageEntity;
import com.crm.rely.backend.core.entity.risk.symbol.RiskSymbolUpdateEntity;
import com.crm.rely.backend.core.entity.risk.symbol.record.*;
import org.springframework.cloud.netflix.feign.FeignClient;
import org.springframework.web.bind.annotation.PostMapping;

@FeignClient(name = "RISKBBI-SERVICE", path = "/risk/symbol/manager")
public interface RiskSymbolFeignService {

    @PostMapping("/add")
    BaseResultDto add(RiskSymbolAddEntity entity) throws Exception;

    @PostMapping("/update")
    BaseResultDto update(RiskSymbolUpdateEntity entity) throws Exception;

    @PostMapping("/delete")
    BaseResultDto delete(RiskSymbolDeleteEntity entity) throws Exception;

    @PostMapping("/search/list")
    ResultWithPagerDto<RiskSymbolDto> searchPage(RiskSymbolSearchPageEntity entity) throws Exception;

    @PostMapping("/search/single")
    BaseResultDto<RiskSymbolDto> searchSingle(SingleLongEntity entity) throws Exception;

    @PostMapping("/add/record/batch")
    BaseResultDto addRecordBatch(RiskSymbolRecordAddBatchEntity entity) throws Exception;

    @PostMapping("/add/record/single")
    BaseResultDto addRecordSingle(RiskSymbolRecordAddSingleEntity entity) throws Exception;

    @PostMapping("/update/record")
    BaseResultDto updateRecord(RiskSymbolRecordUpdateEntity entity) throws Exception;

    @PostMapping("/delete/record")
    BaseResultDto deleteRecord(RiskSymbolRecordDeleteEntity entity) throws Exception;

    @PostMapping("/search/list/record")
    ResultWithPagerDto<RiskSymbolRecordDto> searchRecordPage(RiskSymbolRecordSearchPageEntity entity) throws Exception;

    @PostMapping("/search/single/record")
    BaseResultDto<RiskSymbolRecordDto> searchRecordSingle(SingleLongEntity entity) throws Exception;

}
